#include <FApp.h>
#include <FNet.h>
#include <FWebJson.h>
#include "eyrieMainForm.h"
#include "Codegen.h"

using namespace Tizen::Base;
using namespace Tizen::Base::Utility;
using namespace Tizen::App;
using namespace Tizen::Ui;
using namespace Tizen::Ui::Controls;
using namespace Tizen::Ui::Scenes;
using namespace Tizen::Io;
using namespace Tizen::Media;
using namespace Tizen::Net::Http;
using namespace Tizen::Web::Json;


eyrieMainForm::eyrieMainForm(void)
{
}

eyrieMainForm::~eyrieMainForm(void)
{
}

bool
eyrieMainForm::Initialize(void)
{
	Construct(L"IDF_FORM");

	return true;
}

result
eyrieMainForm::OnInitializing(void)
{
	result r = E_SUCCESS;

    httpSession = new HttpSession();

    int minBufferSize;
    AudioInputDevice inputDevice = AUDIO_INPUT_DEVICE_MIC;
    AudioSampleType sampleType = AUDIO_TYPE_PCM_U8;
    AudioChannelType channelType = AUDIO_CHANNEL_TYPE_MONO;
    int sampleRate = 11025;

    // Constructs an AudioIn instance
    r = audioIn.Construct(*this);
    if (IsFailed(r))
    {
        return r;
    }

    // Prepares the AudioIn instance
    r = audioIn.Prepare(inputDevice, sampleType, channelType, sampleRate);
    if (IsFailed(r))
    {
        return r;
    }

    minBufferSize = audioIn.GetMinBufferSize();

    for (int i = 0; i < MAX_BUFFER_COUNT; i++)
    {
        // Prepares buffers to store PCM data
        r = audioBuffer[i].Construct(sampleRate * 10);
        if (IsFailed(r))
        {
            return r;
        }
        // Adds buffer to the __audioIn
        audioIn.AddBuffer(&audioBuffer[i]);
        if (IsFailed(r))
        {
            return r;
        }
    }

	// Setup back event listener
	SetFormBackEventListener(this);

	// Get a button via resource ID
	Tizen::Ui::Controls::Button *pButtonOk = static_cast<Button*>(GetControl(L"IDC_BUTTON_OK"));
	if (pButtonOk != null)
	{
		pButtonOk->SetActionId(ID_BUTTON_OK);
		pButtonOk->AddActionEventListener(*this);
	}

	return r;
}

result
eyrieMainForm::OnTerminating(void)
{
	result r = E_SUCCESS;

	// TODO:
	// Add your termination code here
	return r;
}

void
eyrieMainForm::OnActionPerformed(const Tizen::Ui::Control& source, int actionId)
{
	SceneManager* pSceneManager = SceneManager::GetInstance();
	AppAssert(pSceneManager);

	switch(actionId)
	{
	case ID_BUTTON_OK:
		AppLog("Record button is clicked!");
		audioIn.Start();
		break;

	default:
		break;
	}
}

void
eyrieMainForm::OnFormBackRequested(Tizen::Ui::Controls::Form& source)
{
	UiApp* pApp = UiApp::GetInstance();
	AppAssert(pApp);
	pApp->Terminate();
}

void
eyrieMainForm::OnSceneActivatedN(const Tizen::Ui::Scenes::SceneId& previousSceneId,
										  const Tizen::Ui::Scenes::SceneId& currentSceneId, Tizen::Base::Collection::IList* pArgs)
{
	// TODO:
	// Add your scene activate code here
	AppLog("OnSceneActivatedN");
}

void
eyrieMainForm::OnSceneDeactivated(const Tizen::Ui::Scenes::SceneId& currentSceneId,
										   const Tizen::Ui::Scenes::SceneId& nextSceneId)
{
	// TODO:
	// Add your scene deactivate code here
	AppLog("OnSceneDeactivated");
}


void eyrieMainForm::OnAudioInBufferIsFilled(Tizen::Base::ByteBuffer* pData)
{
    // Consumes filled data here
	AppLog("Buffer filled");
	FloatBuffer *fbuf = pData->AsFloatBufferN();
	int bufSize = fbuf->GetRemaining();
	float *rawbuf = (float *) malloc(bufSize * sizeof(float));
	//fbuf->GetArray(rawbuf, 0, bufSize);
	for(int i = 0; i < bufSize; i++) {
		byte val;
		pData->GetByte(val);
		// Convert U8 to float
		rawbuf[i] = (val - 128) / 128.0f;
		AppLog("Buf: %f", rawbuf[i]);
	}
	Codegen *codegen = new Codegen(rawbuf, bufSize, 0);
	AppLog("Code: %s", codegen->getCodeString().c_str());

    httpSession->Construct(NET_HTTP_SESSION_MODE_NORMAL, null, L"http://developer.echonest.com/", null);
    HttpTransaction *httpTransaction = httpSession->OpenTransactionN();
    httpTransaction->AddHttpTransactionListener(*this);
    // Get the request instance.
    HttpRequest* httpRequest = httpTransaction->GetRequest();
    // Set the method.
    NetHttpMethod method = NET_HTTP_METHOD_POST;
    httpRequest->SetMethod(method);
    // Set the uri.
    httpRequest->SetUri("http://developer.echonest.com/api/v4/song/identify");
    String querystr("api_key=INSERTYOURKEYHERE&query=[{\"metadata\":{\"version\":4.12},\"code\":\"");
    querystr.Append(codegen->getCodeString().c_str());
    querystr.Append("\"}]");
    ByteBuffer *queryBuf = StringUtil::StringToUtf8N(querystr);
    ByteBuffer body;
    body.Construct(strlen((const char *) queryBuf->GetPointer()));
    body.SetArray((byte *) queryBuf->GetPointer(), 0, strlen((const char *) queryBuf->GetPointer()));
    //AppLog("Querystr: %s", (const char*) queryBuf->GetPointer());
    body.Flip();
    httpRequest->WriteBody(body);
    HttpHeader *pHeader = httpRequest->GetHeader();
    pHeader->AddField(L"Content-Type", L"application/x-www-form-urlencoded;charset=UTF-8");
    httpTransaction->Submit();

    // Clears the buffer
    pData->Clear();

    // Adds the byte buffer to the __audioIn to fill data again
    audioIn.AddBuffer(pData);
}

void eyrieMainForm::OnTransactionCompleted(HttpSession& httpSession, HttpTransaction& httpTransaction) {
	AppLog("Transaction completed");
	HttpResponse* httpResponse = httpTransaction.GetResponse();
	ByteBuffer* buffer = httpResponse->ReadBodyN();
	AppLog("Response: %s", (const char*) buffer->GetPointer());

	IJsonValue *json = JsonParser::ParseN(buffer->GetPointer(), buffer->GetLimit());
	JsonObject *jsonObj = static_cast<JsonObject*>(json);
	httpSession.CloseTransaction(httpTransaction);
}

